﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Časování
{
    public delegate void Načasováno(object sender, EventArgs e);

    internal class Časovač
    {
        public uint Interval { get; set; } = 10;
        public event Načasováno? Tik;
        private bool povolen = false;
        private readonly Thread podproces;
        private readonly object zámek = new();
        public Časovač(uint interval)
        {
            Interval = interval;
            podproces = new Thread(Běh)
            {
                IsBackground = true
            };
            podproces.Start();
        }
        public bool Povolen
        {
            get => povolen;
            set
            {
                lock (zámek)
                {
                    povolen = value;
                    if (povolen)
                    {
                        Monitor.Pulse(zámek);
                    }
                }
            }
        }
        protected virtual void OnTik(EventArgs e) => Tik?.Invoke(this, e);
        private void Běh()
        {
            for (; ; )
            {
                if (Tik != null)
                {
                    OnTik(new EventArgs());
                }
                Thread.Sleep((int)Interval);
                lock(zámek)
                {
                    while(!povolen)
                    {
                        Monitor.Wait(zámek);
                    }
                }
            }
        }
    }
}